function OrgTrabController(templateManager, ScreenService, Utilities, restEngine, General, localStorageEngine, CRM_Utilities, Messages) {
    
    self = this;
    
    this.saveConfiguration = function(args) {
        var widget = templateManager.container.getActiveWidget();
    
        if (widget.isValid()) {
            var paramsRestEngine = {
                "requestType": "Row",
                "row": widget.currentRow
            };
    
            restEngine.post('/saveConfiguration', paramsRestEngine).then(function(response) {
                if ((response.dataset && response.dataset.error) || Util.isEmptyOrBlank(response)) {
                    ScreenService.showMessage(response.dataset.error[0], "ERROR");
                } else if (response.dataset) {
                    ScreenService.successNotification("Parâmetros salvos com sucesso!", 10000);
                    if (widget.currentRow.CRM_NRORGTRAB !== getLocalVar('CRM_NRORGTRABALHO')) {
                        setLocalVar('CRM_NRORGTRABALHO', widget.currentRow.CRM_NRORGTRAB);
                        var userInfo = widget.currentRow.NMORGTRAB + ' - ' + widget.currentRow.CRM_NRORGTRAB;
                        ScreenService.setHeaderInfo(userInfo);
                        ScreenService.buildUserData(templateManager.LOGGEDOPERATOR.NMOPERADOR, [userInfo], function() {}, './assets/images/user.jpg');
    
                        Object.keys(templateManager.containers).forEach(function(container, index) {
                            if (container !== "zeedhi_project" && container !== "product#dashboard" && container !== "product#par3002_paramconfig") {
                                templateManager.historyItems.forEach(function(item, index) {
                                    if (item == container) {
                                        templateManager.historyItems.splice(index, 1);
                                    }
                                });
                                delete templateManager.containers[container];
                            }
                        });
                    }
                    templateManager.updateTemplate();
                    widget.reload().then(function() {
                        Utilities.changeTemplateToView(args);
                    });
                }
            }.bind(this)).catch(function(e) {
                var error;
                if (e.data.error === undefined) {
                    error = e;
                } else {
                    error = e.data.error;
                }
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
            });
        }
    };
    
    this.setDataInCurrentRow = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.currentRow = widget.dataSource.data[0];
    };
    
    this.saveOperators = function() {
        var widget = templateManager.container.getActiveWidget();
        if(widget.isValid()) {
            var rows = [];
            widget.currentRow.CDOPERADOR.forEach(function(operador, index) {
                rows.push(this.setOperatorsInRow(widget, index));
            }.bind(this));
            
            General.callRestEngine('product#/saveRelatedOperators', "DataSet", rows).then(function(response) {
                 if((response.dataset && response.dataset.error) || Util.isEmptyOrBlank(response)) {
                    ScreenService.showMessage(response.dataset.error[0], "ERROR");
                } else if (response.dataset) {
                    ScreenService.successNotification("Operador(es) salvo(s) com sucesso!", 10000);
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload();
                } else {
                    ScreenService.showMessage("Erro ao tentar salvar o registro!<br>Por favor, contate um administrador!", "ERROR");
                }
            }.bind(this)).catch(function(e) {
                var error;
                if(e.data.error === undefined) {
                    error = e;
                }else {
                    error = e.data.error;
                }
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
            });
        }
    };
    
    this.setOperatorsInRow = function(widget, index){
        var currentRow = angular.copy(widget.currentRow);

        currentRow.__is_new = true;
        currentRow.CDOPERADOR = currentRow.CDOPERADOR[index];
        currentRow.NMOPERADOR = currentRow.NMOPERADOR[index];
        currentRow.NRORGTRAB = widget.getParent().currentRow.NRORGTRAB;
        currentRow.NRORG = widget.getParent().currentRow.NRORG;

        return currentRow;
    };
    

    this.deleteOperators = function() {
        ScreenService.confirmMessage('Deseja realmente excluir os operadores selecionados?', 'CONFIRMATION', function(){
            var widget = templateManager.container.getActiveWidget();
            var params;
            var multipleTargets = false;
            
            if(!Util.isEmptyOrBlank(widget.getCheckedRows())) {
                params = widget.getCheckedRows();
                multipleTargets = true;
            } else {
                params = widget.currentRow;
                multipleTargets = false;
            }
            restEngine.post('product#/deleteOperators', {'requestType':'DataSet','dataset':[params]}).
                then(function(data){
                    if(data.messages){
                        ScreenService.successNotification(Messages.REMOVE_ROW_SUCCESS, 10000);
                        if(multipleTargets) {
                            widget.unSelectAll();
                        } else {
                            ScreenService.closeSwipe();
                        }
                        widget.reload();
                    } else {
                        ScreenService.showMessage(Messages.REMOVE_ROW_ERROR);
                    }
                }).catch(function(e) {
                    var error;
                    if(e.data.error === undefined) {
                        error = e;
                    }else {
                        error = e.data.error;
                    }
                    ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
                });
            }
        );
    };
    
    this.setOperatorFilter = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.dataSourceFilter[1].value = widget.getParent().currentRow.NRORGTRAB;
        widget.reload();
    
        var dataSourceFilter = widget.getField('CDOPERADOR').dataSourceFilter;
        var filterItem = null;
        for (var i = 0; i < dataSourceFilter.length; i++) {
            if (dataSourceFilter[i].name === 'P_CRM_NRORGTRAB') {
                filterItem = dataSourceFilter[i];
                break;
            }
        }
        if (filterItem) {
            filterItem.value = widget.getParent().currentRow.NRORGTRAB;
        }
    };
    
    this.widgetOnEnterOnce = function(widget, fieldsToApply) {
        General.callRestEngine("product#/verifyMainOrg", "DataSet", []);
        
        var filterRow = {};
        filterRow.IDATIVO = 'S';
        
        CRM_Utilities.openFilterOnEnter(widget, filterRow, fieldsToApply);
    };
    
    this.goBackToMainGrid = function() {
        var mainWidget = templateManager.container.getWidget('gridOrgOper');
        
        ScreenService.closeSwipe();
        mainWidget.activate();
        mainWidget.list();
    };
	
}

Configuration(function(ContextRegister) {
    ContextRegister.register('OrgTrabController', OrgTrabController);
});