function crm00001002_Operador_Perfil_Atividade_Controller(General, CRM_Utilities, ScreenService) {
    
    /**
     * Função para salvar um operador ao perfil
     * de atividade em caso.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do operador.
    */
    this.saveNewActivityProfileOperator = function(widget){
        
        var currentRow = widget.currentRow;
        
        var activityProfileOperators = currentRow.OPERADORES;
        var rows = [];
        
        activityProfileOperators.forEach(function(operator, operatorIndex){
            
            rows.push(this.setActivityProfileOperatorValuesInCurrentRow(widget, operatorIndex));
                
        }.bind(this)); 
        
        General.callRestEngine("product#/crm_perfioper_operador/save", "DataSet", rows).then(function(response){
                
                ScreenService.closeSwipe().then(function(){
                    
                    widget.reload();
                    widget.getField("OPERADORES").reload();
                    
                });
                
        }.bind(this));
    };
    
    /**
     * Função para pegar as informações do 
     * array de operadores na currentRow, e
     * tratá-las em uma cópia da currentRow para
     * realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do operador.
     * 
     * @param {int} operatorIndex index do operador
     * no array de operadores presente na currentRow.
    */
    this.setActivityProfileOperatorValuesInCurrentRow = function(widget, operatorIndex){
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.__is_new = true;
        currentRow.NRSEQPERFATIV = widget.parent.parent.currentRow.NRSEQPERFATIV;
        currentRow.CDOPERADOR = currentRow.OPERADORES[operatorIndex];
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        
        return currentRow;
        
    };
    
    /**
     * Define as regras do campo de operador na aba de operadores
     * na widget perfil de atividade de acordo 
     * com a revenda do operador logado.
     * 
     * @param {object} widget widget de operadores
     * ('WDGT_OPER_PERFIL_ATIVIDADE').
    */
    this.setResaleRules = function(widget) {
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        if(loggedOperatorInfos.IDTPOPERADOR === 'R' && Boolean(loggedOperatorInfos.NRREVENDA)){
            
            var restrictObjects = [];
            
            restrictObjects.push(widget.getField("OPERADORES"));
            
            CRM_Utilities.setResaleRulesInRestrictObjectsByLoggedOperatorResale(restrictObjects);
            
        }
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm00001002_Operador_Perfil_Atividade_Controller', crm00001002_Operador_Perfil_Atividade_Controller);
});